/*
DivAgain, un robot para RealTimeBattle
Copyright (C) 2003  Dives

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <math.h>
#include <string.h>

#include "dives.hh"

#define abs(x) ((x>0) ? (x) : -(x))

DATOSROBOT drob;
//GAMEOPTION gopt;

bool enemigo = false;	// hay algn enemigo?
float distEnemigo;		// distancia al enemigo
bool girando = false;	// est girando?
double acel = 0.0;		// aceleracin
//int visto = 0;				// hace cuanto que has visto al enemigo
//#define NUMVISTO 10
int rafaga = 0;
#define MAXRAFAGA 3

/*
 * Enva nombre y color si es la primera sequencia
 */
void initialize( const int first_seq )
{
	if (first_seq == 1)
	{
		name("DivAgain");
		colour("FF0000", "999999");
	}
}

/*
 * Resetea el robot cuando empieza un nuevo juego
 */
void gameStarts()
{
	acel = 2.0;
	accelerate(acel);
	rotate(RCANNON+RRADAR, M_PI);
	rotate(RROBOT, M_PI_2);	
}

/*
 * Recibe informacin sobre tiempo, velocidad y ngulo del can
 */
void info( const double tiempo, const double speed, const double cannon_angle )
{	
	if (enemigo && (rafaga < MAXRAFAGA))
	{
		float energia = 30.0 / (distEnemigo > 5 ? 2:1);
		shoot(energia);
		rafaga++;		
	}
	drob.speed=speed;
}

/*
 * El nmero de robots que quedan en el juego
 */
void robotsLeft( const int number_of_robots )
{
	drob.robotsLeft = number_of_robots;
}

/*
 * Radar: informacin cuando se ve un robot
 */
void radarRobot( const double dist, const double angulo )
{
	rafaga = 0;
	distEnemigo = dist;
	
	float energy = 32 / (dist > 5 ? 2:1);
	shoot(energy);
	//shoot(32/(dist > 5? 2:1));
	sweep(RCANNON+RRADAR, M_PI, -M_PI/10.0, M_PI/10.0);
	enemigo = true;
	alarm(2);
	girando = true;
	rotateAmount(RROBOT, M_PI, angulo);
	if (drob.speed < 5.0)
		acel = 2.0;
	else
		acel = 0;
}

/*
 * Radar: informacin cuando se ve una pared
 */
void radarWall( const double dist, const double angulo )
{	
	if (dist < 1.0)
	{
		if (drob.speed > 0.1)
		{
			acel = 0.0;
			brake(1.0);
		}
		else
		{
			acel = 0.1;
		}
	}
	else if (dist < 4.0)
	{
		if (drob.speed > 2.0)
		{
			acel = 0.0;
			brake(0.7);
		}
		else
		{
			acel = 0.1;
		}
	}
	else
	{
		acel = 0.1;
	}
}

/*
 * Radar: informacin cuando se ve un disparo
 */
void radarShot( const double dist, const double angulo )
{
	//***************************************************************************
}

/*
 * Radar: informacin cuando se ve una galleta
 */
void radarCookie( const double dist, const double angulo )
{
	if (!enemigo && !girando)
	{		
		rotateAmount(RROBOT, M_PI/4.1, angulo);
		if (dist > 3)
			accelerate(1.5);
	}
}

/*
 * Radar: informacin cuando se ve una mina
 */
void radarMine( const double dist, const double angulo )
{
	if (dist < 3)
	{
		shoot (0.5);
		if (!enemigo && !girando)
			rotate(RROBOT, M_PI/4.1);
	}
}

/*
 * Colisin contra otro robot
 */
void collisionRobot( const double angulo )
{
	//***************************************************************************
}

/*
 * Colisin contra una pared
 */
void collisionWall( const double angulo )
{
	if (!enemigo && !girando)
	{
		rotateAmount(RROBOT, M_PI/4.1, M_PI/2.0);
		accelerate(1.5);
	}
}

/*
 * Colisin contra un disparo
 */
void collisionShot( const double angulo )
{
	//char mens[100];
	//sprintf(mens, "Angulo disparo: %f", angulo);
	//debug(mens);
	
	// Cmo coo funciona este ngulo!?!?!?
	if (!enemigo)
	{
		if (abs(angulo) < 5)
			rotateAmount(RROBOT, M_PI/4.1, -M_PI/2.0);
		accelerate(2.0);
	}
}

/*
 * Colisin contra una galleta
 */
void collisionCookie( const double angulo )
{
	print("Una deliciosa galletita! :)");
}

/*
 * Colisin contra una mina
 */
void collisionMine( const double angulo )
{
	print("Mier**! Una mina! :(");
}

/*
 * Mensaje de advertencia
 */
void warning( const int tipo, const char* men )
{
	char mensaje[200];

	switch(tipo)
	{
		case UNKNOWN_MESSAGE:
			strcpy(mensaje, "Unknown message: ");
			break;
		case PROCESS_TIME_LOW:
			strcpy(mensaje, "Process time low: ");
			break;
		case MESSAGE_SENT_IN_ILLEGAL_STATE:
			strcpy(mensaje, "Message sent in illegal state: ");
			break;
		case UNKNOWN_OPTION:
			strcpy(mensaje, "Unknown option: ");
			break;
		case OBSOLETE_KEYWORD:
			strcpy(mensaje, "Obsolete Keyword: ");
			break;
	}
	strcat(mensaje, men);
	print(mensaje);
}

/*
 * Hay que sacar el robot
 */
void exitRobot()
{
	print("Me voy, hasta luego!");
	exit(0);
}

/*
 * El nombre que el robot tiene en el juego
 */
void yourName( const char* prev_name )
{
	//***************************************************************************
}

/*
 * El color que el robot tiene en el juego
 */
void yourColour( const char* color )
{
	//***************************************************************************
}

/*
 * Recibe datos sobre una opcin del juego
 */
void gameOption( const int opcion, const double valor )
{
/*
	switch(opcion)
	{
		case ROBOT_MAX_ROTATE:
			gopt.robot_max_rotate = valor;
			break;
		case ROBOT_CANNON_MAX_ROTATE:
			gopt.robot_cannon_max_rotate = valor;
			break;
		case ROBOT_RADAR_MAX_ROTATE:
			gopt.robot_radar_max_rotate = valor;
			break;
		case ROBOT_MAX_ACCELERATION:
			gopt.robot_max_acceleration = valor;
			break;
		case ROBOT_MIN_ACCELERATION:
			gopt.robot_min_acceleration = valor;
			break;
		case ROBOT_START_ENERGY:
			gopt.robot_start_energy = valor;
			break;
		case ROBOT_MAX_ENERGY:
			gopt.robot_max_energy = valor;
			break;
		case ROBOT_ENERGY_LEVELS:
			gopt.robot_energy_levels = valor;
			break;
		case SHOT_SPEED:
			gopt.shot_speed = valor;
			break;
		case SHOT_MIN_ENERGY:
			gopt.shot_min_energy = valor;
			break;
		case SHOT_MAX_ENERGY:
			gopt.shot_max_energy = valor;
			break;
		case SHOT_ENERGY_INCREASE_SPEED:
			gopt.shot_energy_increase_speed = valor;
			break;
		case TIMEOUT:
			gopt.timeout = valor;
			break;
		case DEBUG_LEVEL:
			gopt.debug_level = valor;
			break;
		case SEND_ROBOT_COORDINATES:
			gopt.send_robot_coordinates = (int)rint(valor);
			break;
	}
*/
}

/*
 * Informacin sobre la posicin del robot.
 * Las coordenadas pueden ser absolutas, relativas a la posicin de comienzo,
 * o nada, dependiendo de la opcin SEND_ROBOT_COORDINATES
 */
void coordinates( const double x, const double y, const double rotacion)
{
	//***************************************************************************
}

/*
 * Informacin sobre el robot avistado
 */
void robotInfo( const double energia, int enemigo )
{
	//***************************************************************************
}

/*
 * Cuando un giro ha concluido
 */
void rotationReached( const int que )
{
	accelerate(acel);
	if (!enemigo)
		rotate(RCANNON+RRADAR, M_PI);
	girando = false;
}

/*
 * El nivel de energa del robot
 */
void energy( const double nivEnergia )
{
	//***************************************************************************
}

/*
 * El juego ha terminado
 */
void gameFinishes()
{
	//***************************************************************************
}

/*
 * El robot est muerto
 */
void dead()
{
	//print("Me han matado!");
}

/********************** OTRAS FUNCIONES ***************************************/
void handle_alarm(int sig)
{
	enemigo = false;
}
